==> klayout-0.21.5.tar.gz: klayout-0.21.5/src/dbCIF.cc <==

/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include "dbCIF.h"
#include "dbCIFReader.h"
#include "dbCIFWriter.h"
#include "dbStream.h"

#include "tlClassRegistry.h"

namespace db
{

// ---------------------------------------------------------------
//  CIFDiagnostics implementation

CIFDiagnostics::~CIFDiagnostics ()
{
  //  .. nothing yet ..
}

// ---------------------------------------------------------------
//  CIF format declaration

class CIFFormatDeclaration
  : public db::StreamFormatDeclaration
{
public:
  CIFFormatDeclaration ()
  {
    //  .. nothing yet ..
  }

  virtual std::string format_name () const { return "CIF"; }
  virtual std::string format_desc () const { return "CIF stream format"; }
  virtual std::string file_format () const { return "CIF files (*.CIF *.cif *.cif.gz *.CIF.gz)"; }

  virtual bool detect (tl::InputStreamBase &stream_base) const 
  {
    try {

      std::string cmd;
      tl::ASCIIInputStream stream (stream_base);

      while (! stream.at_end ()) {

        cmd.clear ();

        while (! stream.at_end () && stream.peek_char () != ';') {
          
          // skip over comments
          if (stream.peek_char () == '(') {

            stream.get_char ();

            char c;
            int bl = 0;
            while (! stream.at_end () && ((c = stream.get_char ()) != ')' || bl > 0)) {
              // check for nested comments (bl is the nesting level)
              if (c == '(') {
                ++bl;
              } else if (c == ')') {
                --bl;
              }
            }

          } else {
            cmd += stream.get_char ();
          }

        }

        if (stream.peek_char () == ';') {
          stream.get_char ();
        }

        //  The first command must be a DS command (DS n ...)
        tl::Extractor ex (cmd.c_str ());
        unsigned int n;
        if (! ex.at_end () && ((ex.test ("DS ") && ex.try_read (n)) || ex.test ("L"))) {
          return true;
        }

      }

    } catch (...) {
    }

    return false;
  }

  virtual ReaderBase *create_reader (tl::InputStreamBase &s) const 
  {
    return new db::CIFReader (s);
  }

  virtual WriterBase *create_writer () const
  {
    return new db::CIFWriter ();
  }

  virtual bool can_read () const
  {
    return true;
  }

  virtual bool can_write () const
  {
    return true;
  }
};

static tl::RegisteredClass<db::StreamFormatDeclaration> reader_decl (new CIFFormatDeclaration (), 100, "CIF");

//  provide a symbol to force linking against
int force_link_CIF = 0;

}

==> klayout-0.21.19.tar.gz: klayout-0.21.19/src/dbCIF.cc <==

/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include "dbCIF.h"
#include "dbCIFReader.h"
#include "dbCIFWriter.h"
#include "dbStream.h"

#include "tlClassRegistry.h"

namespace db
{

// ---------------------------------------------------------------
//  CIFDiagnostics implementation

CIFDiagnostics::~CIFDiagnostics ()
{
  //  .. nothing yet ..
}

// ---------------------------------------------------------------
//  CIF format declaration

class CIFFormatDeclaration
  : public db::StreamFormatDeclaration
{
public:
  CIFFormatDeclaration ()
  {
    //  .. nothing yet ..
  }

  virtual std::string format_name () const { return "CIF"; }
  virtual std::string format_desc () const { return "CIF stream format"; }
  virtual std::string file_format () const { return "CIF files (*.CIF *.cif *.cif.gz *.CIF.gz)"; }

  virtual bool detect (tl::InputStreamBase &stream_base) const 
  {
    try {

      std::string cmd;
      tl::ASCIIInputStream stream (stream_base);

      while (! stream.at_end ()) {

        cmd.clear ();

        while (! stream.at_end () && stream.peek_char () != ';') {
          
          // skip over comments
          if (stream.peek_char () == '(') {

            stream.get_char ();

            char c;
            int bl = 0;
            while (! stream.at_end () && ((c = stream.get_char ()) != ')' || bl > 0)) {
              // check for nested comments (bl is the nesting level)
              if (c == '(') {
                ++bl;
              } else if (c == ')') {
                --bl;
              }
            }

          } else {
            cmd += stream.get_char ();
          }

        }

        if (stream.peek_char () == ';') {
          stream.get_char ();
        }

        //  The first command must be a DS or L command (DS n ... or L n)
        tl::Extractor ex (cmd.c_str ());
        unsigned int n;
        if (! ex.at_end ()) {
          if ((ex.test ("DS ") && ex.try_read (n)) || ex.test ("L ")) {
            return true;
          } else if (! ex.test ("9")) {
            return false;
          }
        } 

      }

    } catch (...) {
    }

    return false;
  }

  virtual ReaderBase *create_reader (tl::InputStreamBase &s) const 
  {
    return new db::CIFReader (s);
  }

  virtual WriterBase *create_writer () const
  {
    return new db::CIFWriter ();
  }

  virtual bool can_read () const
  {
    return true;
  }

  virtual bool can_write () const
  {
    return true;
  }
};

static tl::RegisteredClass<db::StreamFormatDeclaration> reader_decl (new CIFFormatDeclaration (), 100, "CIF");

//  provide a symbol to force linking against
int force_link_CIF = 0;

}
